==> orthanc_1.3.1+dfsg.orig.tar.xz: Orthanc-1.3.1/Core/MultiThreading/Semaphore.cpp <==
/**
 * Orthanc - A Lightweight, RESTful DICOM Store
 * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
 * Department, University Hospital of Liege, Belgium
 * Copyright (C) 2017 Osimis, Belgium
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * In addition, as a special exception, the copyright holders of this
 * program give permission to link the code of its release with the
 * OpenSSL project's "OpenSSL" library (or with modified versions of it
 * that use the same license as the "OpenSSL" library), and distribute
 * the linked executables. You must obey the GNU General Public License
 * in all respects for all of the code used other than "OpenSSL". If you
 * modify file(s) with this exception, you may extend this exception to
 * your version of the file(s), but you are not obligated to do so. If
 * you do not wish to do so, delete this exception statement from your
 * version. If you delete this exception statement from all source files
 * in the program, then also delete it here.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 **/


#include "../PrecompiledHeaders.h"
#include "Semaphore.h"

#include "../OrthancException.h"


namespace Orthanc
{
  Semaphore::Semaphore(unsigned int count) : count_(count)
  {
  }

  void Semaphore::Release()
  {
    boost::mutex::scoped_lock lock(mutex_);

    count_++;
    condition_.notify_one(); 
  }

  void Semaphore::Acquire()
  {
    boost::mutex::scoped_lock lock(mutex_);

    while (count_ == 0)
    {
      condition_.wait(lock);
    }

    count_++;
  }
}

==> orthanc_1.5.8+dfsg.orig.tar.xz: Orthanc-1.5.8/Core/MultiThreading/Semaphore.cpp <==
/**
 * Orthanc - A Lightweight, RESTful DICOM Store
 * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
 * Department, University Hospital of Liege, Belgium
 * Copyright (C) 2017-2019 Osimis S.A., Belgium
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * In addition, as a special exception, the copyright holders of this
 * program give permission to link the code of its release with the
 * OpenSSL project's "OpenSSL" library (or with modified versions of it
 * that use the same license as the "OpenSSL" library), and distribute
 * the linked executables. You must obey the GNU General Public License
 * in all respects for all of the code used other than "OpenSSL". If you
 * modify file(s) with this exception, you may extend this exception to
 * your version of the file(s), but you are not obligated to do so. If
 * you do not wish to do so, delete this exception statement from your
 * version. If you delete this exception statement from all source files
 * in the program, then also delete it here.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 **/


#include "../PrecompiledHeaders.h"
#include "Semaphore.h"

#include "../OrthancException.h"


namespace Orthanc
{
  Semaphore::Semaphore(unsigned int availableResources) :
    availableResources_(availableResources)
  {
    if (availableResources_ == 0)
    {
      throw OrthancException(ErrorCode_ParameterOutOfRange);
    }
  }

  void Semaphore::Release()
  {
    boost::mutex::scoped_lock lock(mutex_);

    availableResources_++;
    condition_.notify_one(); 
  }

  void Semaphore::Acquire()
  {
    boost::mutex::scoped_lock lock(mutex_);

    while (availableResources_ == 0)
    {
      condition_.wait(lock);
    }

    availableResources_--;
  }

  bool Semaphore::TryAcquire()
  {
    boost::mutex::scoped_lock lock(mutex_);

    if (availableResources_ == 0)
    {
      return false;
    }

    availableResources_--;
    return true;
  }
}
